<?php
	if (isset($_GET['version'])) {
		include_once('version.php');
	}
	ob_start();
	include_once('auth.php');
	$emailCache = array();

	/**
	 * Queries the database to generate a mapping of bean ids to email addresses.
	 *
	 * @param array $emails
	 * @param string $module one of 'Contacts' or 'Leads'
	 * @return array a mapping of bean ids to email addresses
	 */
	function getBeanIdsFromEmails($emails, $module, $includeDeleted = false) {
		//sanitize module
		if (!in_array($module, array('Contacts', 'Leads'))) {
			serializeError('Invalid module');
		}
		//sanitize emails
		$emails = sanitizeEmails($emails);
		
		global $db;
		global $emailCache;
		$serializedEmails = serialize($emails);
		if (!isset($emailCache[$serializedEmails])) {
			$emailMap = array();
			foreach (array_chunk($emails, 200, true) as $emailChunk) {
				$queryResult = $db->query('SELECT id, email_address FROM email_addresses WHERE email_address in (\''.implode('\', \'', $emailChunk).'\')');
				while (($queryRow=$db->fetchByAssoc($queryResult)) != null) {
					$emailMap[$queryRow['id']] = $queryRow['email_address'];
				}
			}
			$emailCache[$serializedEmails] = $emailMap;
		} else {
			$emailMap = $emailCache[$serializedEmails];
		}
		
		$beanIdToEmailMap = array();
		$deletedFilter = (!$includeDeleted) ? 'deleted = 0 AND' : '';
		foreach (array_chunk($emailMap, 200, true) as $emailMapChunk) {
			$queryResult = $db->query('SELECT bean_id, email_address_id FROM email_addr_bean_rel WHERE '.$deletedFilter.' bean_module = \''.$module.'\' AND email_address_id in (\''.implode('\', \'', array_keys($emailMapChunk)).'\')');
			while (($queryRow=$db->fetchByAssoc($queryResult)) != null) {
				$beanIdToEmailMap[$queryRow['bean_id']] = $emailMapChunk[$queryRow['email_address_id']];
			}
		}
		return $beanIdToEmailMap;
	}

	/**
	 * Queries the database to generate a mapping of bean ids to email addresses.
	 *
	 * @param array $emails
	 * @param string $module one of 'Contacts' or 'Leads'
	 * @return array a mapping of bean ids to email addresses
	 */
	function getModifiedBeans($datetime, $module, $fields = null, $limit = null, $offset = null) {
		//sanitize module
		if (!in_array($module, array('Contacts', 'Leads', 'Opportunities', 'opportunities_contacts', 'opportunities_leads', 'Campaigns'))) {
			serializeError('Invalid module');
		}

		if (!$datetime) {
			serializeError('Invalid datetime');
		} else {
			$timestamp = strtotime($datetime);
			if (date('Y-m-d H:i:s', $timestamp) != $datetime) {
				serializeError('Invalid datetime');
			}
		}
		
		if (!$fields) {
			$fields = array('date_modified');
		}
		$fields []= 'id';

		if ($limit !== null || $offset !== null) {
			$limit = (int)$limit;
			$offset = (int)$offset;
			if ($limit < 0 || $offset < 0) {
				serializeError("Invalid limit or offset");
			}
		}
		
		$table = strtolower($module);
		
		global $db;
		$beanMap = array();
		$fieldSelect = implode(', ', $fields);
		if ($limit === null) {
				$query = 'SELECT '.$fieldSelect.' FROM '.$table.' WHERE date_modified > \''.$datetime.'\'';
		} else {
			global $sugar_config;
			if ($sugar_config['dbconfig']['db_type'] == 'mssql') {
				// MSSQL has no offset equivalent :(
				$query = 'SELECT  TOP '.$limit.' '.$fieldSelect.' FROM '.$table.' WHERE date_modified > \''.$datetime.'\' AND id NOT IN (SELECT TOP '.$offset.' id FROM '.$table.' WHERE date_modified >\''.$datetime.'\')';
			} else {
				$query = 'SELECT '.$fieldSelect.' FROM '.$table.' WHERE date_modified > \''.$datetime.'\' limit '.$limit.' offset '.$offset;
			}
		}
		$queryResult = $db->query($query);
		while (($queryRow=$db->fetchByAssoc($queryResult)) != null) {
			$beanMap[$queryRow['id']] = $queryRow;
		}
		return $beanMap;
	}

	/**
	 * Queries the database to generate a mapping of bean ids to email addresses.
	 *
	 * @param array $emails
	 * @param string $module one of 'Contacts' or 'Leads'
	 * @return array a mapping of bean ids to email addresses
	 */
	function getEmailsFromBeanIds($beanIds, $module, $includeDeleted = false) {
		//sanitize module

		if (!in_array($module, array('Contacts', 'Leads'))) {
			serializeError('Invalid module');
		}

		global $db;
		$beanMap = array();
		
		if ($beanIds) {
			$deletedFilter = (!$includeDeleted) ? 'deleted = 0 AND' : '';
			foreach (array_chunk($beanIds, 200, true) as $beanIdChunk) {
				$queryResult = $db->query('SELECT bean_id, email_address_id FROM email_addr_bean_rel WHERE '.$deletedFilter.' bean_module = \''.$module.'\' AND bean_id in (\''.implode('\', \'', $beanIdChunk).'\')');
				while (($queryRow=$db->fetchByAssoc($queryResult)) != null) {
					if (!array_key_exists($queryRow['email_address_id'], $beanMap)) {
						$beanMap[$queryRow['email_address_id']] = array();
					}
					$beanMap[$queryRow['email_address_id']] []= $queryRow['bean_id'];
				}
			}
		}

		$emailMap = array();
		if ($beanMap) {
			foreach (array_chunk($beanMap, 200, true) as $beanMapChunk) {
				$queryResult = $db->query('SELECT id, email_address FROM email_addresses WHERE id in (\''.implode('\', \'', array_keys($beanMapChunk)).'\')');
				while (($queryRow=$db->fetchByAssoc($queryResult)) != null) {
					$emailMap[$queryRow['email_address']] = $beanMapChunk[$queryRow['id']];
				}
			}
		}
					
		return $emailMap;
	}
	
	function getNameValueListFromObject($classObject)
	{
		$timeDate = $GLOBALS['timedate'];
		$classObject->fill_in_additional_detail_fields();
		$nameValueList = array();
		if (isset($classObject->assigned_user_name)) {
			$nameValueList['assigned_user_name'] = array('name'=>'assigned_user_name', 'value'=>$classObject->assigned_user_name);
		}
		if (isset($classObject->modified_by_name)) {
			$nameValueList['modified_by_name'] = array('name'=>'modified_by_name', 'value'=>$classObject->modified_by_name);
		}
		if (isset($classObject->created_by_name)) {
			$nameValueList['created_by_name'] = array('name'=>'created_by_name', 'value'=>$classObject->created_by_name);
		}
		foreach ($classObject->field_defs as $fieldDef) {
			if (isset($fieldDef['source']) && ($fieldDef['source'] != 'db' && $fieldDef['source'] != 'custom_fields') && $fieldDef['name'] != 'email1' && $fieldDef['name'] != 'email2' && (!isset($fieldDef['type'])|| $fieldDef['type'] != 'relate')) {
				if($classObject->module_dir == 'Emails' && (($fieldDef['name'] == 'description') || ($fieldDef['name'] == 'description_html') || ($fieldDef['name'] == 'from_addr_name') || ($fieldDef['name'] == 'reply_to_addr') || ($fieldDef['name'] == 'to_addrs_names') || ($fieldDef['name'] == 'cc_addrs_names') || ($fieldDef['name'] == 'bcc_addrs_names') || ($fieldDef['name'] == 'raw_source'))) {

				} else {
					continue;
				}
			}
			if (isset($classObject->$fieldDef['name']) && ($value = $classObject->$fieldDef['name']) !== "") {
				if (strcmp($fieldDef['type'], 'date') == 0){
					$value = $timeDate->swap_formats($value, $timeDate->get_date_format(), 'Y-m-d');
				}
				if (strcmp($fieldDef['type'], 'datetime') == 0){
					$value = $timeDate->swap_formats($value, $timeDate->get_date_time_format(), 'Y-m-d H:i:s');
					$value = $timeDate->handle_offset($value, $timeDate->get_db_date_time_format(), false, $current_user, null);
				}
				$nameValueList[$fieldDef['name']] = array(
					'name'			=> $fieldDef['name'],
					'value'			=> $value,
				);
			}
		}
		return $nameValueList;
	}

	/**
	 * Takes a row from the database and an array of fields and creates an array of name-value pairs
	 * for each field specified.
	 *
	 * @param array $row format: array('abc' => '123', 'def' => ...)
	 * @return array format: array(array('name' => 'abc', 'value' => '123'), array('name' => ...))
	 */
	function generateNameValueList($row)
	{
		$result = array();
		foreach($row as $name => $value) {
			$result[] = array('name' => $name, 'value' => $value);
		}
		return $result;
	}

	/**
	 * Sanitizes an input array of emails, and prepares it for DB querying
	 * @param array $emails
	 */
	function sanitizeEmails($emails)
	{
		global $db;
		if (!($emails && is_array($emails))) {
			serializeError('No emails provided');
		}
		
		$emailRegex = "/^['\.\-\+\/\w]*@((\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})|\w+([\.-]?\w+)*(\.[\w-]{2,}))$/";
		$validEmails = array();
		foreach ($emails as $email) {
			$email = html_entity_decode($email, ENT_QUOTES);
			if (preg_match($emailRegex, $email)) {
				$validEmails[] = $db->quote($email);
			} else {
				serializeError('Invalid email: '.$email);
			}
		}
		return $validEmails;
	}

	function sanitizeUUIDs($uuids)
	{
		if (!($uuids && is_array($uuids))) {
			serializeError('No ids provided');
		}

		$validUUIDs = array();
		foreach ($uuids as $uuid) {
			$sanitizedUUID = sanitizeUUID($uuid);
			if($sanitizedUUID) {
				$validUUIDs[] = $sanitizedUUID;
			}
		}
		return $validUUIDs;
	}

	function sanitizeUUID($uuid)
	{
		if (!$uuid) {
			serializeError('No id provided');
		}

		$validUUID = null;
		$uuidRegex = "/[a-f0-9]{8}(-[a-f0-9]{4}){3}-[a-f0-9]{12}/";
		$uuid = strtolower($uuid);
		if (!preg_match($uuidRegex, $uuid)) {
			serializeError('Invalid id: '.$uuid);
		} else {
			$validUUID = $uuid;
		}
		return $validUUID;
	}
	
	function serializeResult($result)
	{
		ob_end_clean();
		echo serialize($result);
		exit;
	}
	
	function serializeError($err)
	{
		serializeResult('Error: '.$err);
	}

