<?php
	include_once('common.php');

	$memberIds = null;
	$membersType = null;
	$campaignIds = null;
	$datetime = null;
	$limit = null;
	$ignoreMembers = true;

	// Vars for chunking membership data
	$memberLimit = null;
	$lastQueriedProspectIds = array(); // array of from campaignId => lastQueriedProspectId

	// Sanitize all of the possible inputs
	if (isset($_POST['memberIds']) && is_array($_POST['memberIds']) && isset($_POST['type'])) {
		if ($_POST['memberIds']) {
			$memberIds = sanitizeUUIDs($_POST['memberIds']);
		}
		if ($_POST['type'] && !in_array($_POST['type'], array('Contacts', 'Leads'))) {
			serializeError('Invalid record type');
		}
		$membersType = $_POST['type'];
		$campaignIds = getCampaignIdsFromTargetListMembership($memberIds, $membersType);
	} elseif (isset($_POST['campaignIds']) && is_array($_POST['campaignIds'])) {
		$campaignIds = sanitizeUUIDs($_POST['campaignIds']);
	}
	if (isset($_POST['limit']) && is_numeric($_POST['limit'])) {
		$limit = (int)$_POST['limit'];
	}
	if (isset($_POST['since']) && !empty($_POST['since'])) {
		$timestamp = strtotime($_POST['since']);
		if (date('Y-m-d H:i:s', $timestamp) != $_POST['since']) {
			serializeError('Invalid datetime');
		}
		$datetime = $_POST['since'];
		if (!$campaignIds) {
			$campaignIds = getCampaignIdsFromModifiedLists($datetime);
		}
	}
	if (isset($_POST['ignoreMembers'])) {
		$ignoreMembers = ((int)$_POST['ignoreMembers'] === 0) ? false : true;
	}
	if(!$ignoreMembers && isset($_POST['memberLimit']) && is_numeric($_POST['memberLimit'])) {
		$memberLimit = (int) $_POST['memberLimit'];
	}
	if($memberLimit && isset($_POST['lastQueriedProspectIds']) && is_array($_POST['lastQueriedProspectIds'])) {
		$lastQueriedProspectIds = sanitizeLastQueriedProspectIds($_POST['lastQueriedProspectIds']);
	}

	// We don't currently use all of these fields, but let's pull in case we want them in the future.
	$fields = array('id', 'name', 'date_entered', 'date_modified', 'deleted', 'assigned_user_id', 'start_date', 'end_date', 'status', 'budget', 'expected_cost', 'actual_cost', 'expected_revenue', 'campaign_type', 'objective');
	$campaigns = getCampaignData($fields, $campaignIds, $datetime, $limit);
	$campaignIds = array_keys($campaigns);

	if ($ignoreMembers) {
		// Only get campaign data
		$campaignsMetaData = array();
		foreach ($campaignIds as $campaignId) {
			$campaignsMetaData[$campaignId] = array('fields' => $campaigns[$campaignId]);
		}
		serializeResult($campaignsMetaData);
	} elseif((!$memberIds || !$membersType) && $memberLimit){
		// Get campaign and membership data - pull membership data in limited chunks
		$campaignData = getCampaignMembershipDataInChunks($campaignIds, $memberLimit, $lastQueriedProspectIds);
		foreach($campaignIds as $campaignId) {
			if(!isset($campaignData[$campaignId])) {
				$campaignData[$campaignId] = array();
			}
			$campaignData[$campaignId]['fields'] = $campaigns[$campaignId];
		}
		// Returns array of the form: campaignId =>  array(fields => ... prospect_emails => ... lastQueriedProspectId => ... finished => true / false)
		serializeResult($campaignData);
	} else {
		// Get campaign and membership data - pull membership data all at once
		$emailMap = getEmailMapFromCampaigns($campaignIds, $memberIds, $membersType);
		$campaignsAndMembers = array();
		foreach ($campaignIds as $campaignId) {
			// Do not include empty member campaigns OR campaign metadata when querying by memberIds
			if ($memberIds && count($emailMap[$campaignId])) {
				$campaignsAndMembers[$campaignId] = array('prospect_emails' => $emailMap[$campaignId]);
			} elseif (!$memberIds) {
				$campaignsAndMembers[$campaignId] = array('fields' => $campaigns[$campaignId], 'prospect_emails' => $emailMap[$campaignId]);
			}
		}
		serializeResult($campaignsAndMembers);
	}


	/**
	 * Should be of the form: campaignId => prospectBeanId
	 * @param $lastQueriedProspectIds
	 * @return array
	 */
	function sanitizeLastQueriedProspectIds($lastQueriedProspectIds) {
		$sanitized = array();
		foreach($lastQueriedProspectIds as $key => $value) {
			$sanitizedKey = sanitizeUUID($key);
			$sanitizedValue = sanitizeUUID($value);
			if($sanitizedKey && $sanitizedValue) {
				$sanitized[$sanitizedKey] = $sanitizedValue;
			}
		}
		return $sanitized;
	}

	/**
	 * Queries the database for a list of campaigns using a default target list containing the given
	 * $recordIds of $recordType.  This does not filter out disclude lists, which is handled later.
	 *
	 * @param array $recordIds list of members we want to match to campaigns
	 * @param string $relevantType type of record to which $recordIds refers
	 * @return array $campaignIds 
	 */
	function getCampaignIdsFromTargetListMembership($recordIds, $recordType) {
		global $db;

		$campaignIds = array();
		$queryResult = $db->query("SELECT plp.related_id, plc.campaign_id FROM prospect_lists_prospects plp, prospect_lists pl, prospect_list_campaigns plc WHERE plp.prospect_list_id=pl.id AND pl.id=plc.prospect_list_id AND plp.deleted=0 AND pl.deleted=0 AND plc.deleted=0 AND plp.related_type='".$recordType."' AND plp.related_id IN ('".implode('\',\'', $recordIds)."')");
		while (($queryRow=$db->fetchByAssoc($queryResult)) != null) {
			$campaignIds [] = $queryRow['campaign_id'];
		}
		return $campaignIds;
	}
	/**
	 * Queries the database for a list of campaigns using a default target list that have been
	 * modified since $datetime.  This does not filter out disclude lists, which is handled later.
	 *
	 * @param string $datetime modification date cutoff
	 * @return array $campaignIds 
	 */
	function getCampaignIdsFromModifiedLists($datetime) {
		global $db;

		$campaignIds = array();
		$andWhere = " AND (plp.date_modified > '".$datetime."' OR pl.date_modified > '".$datetime."' OR plc.date_modified > '".$datetime."')";
		$queryResult = $db->query("SELECT plp.related_id, plc.campaign_id FROM prospect_lists_prospects plp, prospect_lists pl, prospect_list_campaigns plc WHERE plp.prospect_list_id=pl.id AND pl.id=plc.prospect_list_id".$andWhere);
		while (($queryRow=$db->fetchByAssoc($queryResult)) != null) {
			$campaignIds [] = $queryRow['campaign_id'];
		}
		return $campaignIds;
	}
	/**
	 * Simple query for campaign information.  $campaignIds and $datetime are additive.
	 *
	 * @param array $fields
	 * @param array $campaignIds optional parameter for limiting the campaigns to query
	 * @param array $datetime optional parameter for limiting campaigns by data_modified
	 * @return array $campaigns
	 */
	function getCampaignData($fields, $campaignIds = null, $datetime = null, $limit = null) {
		global $db, $sugar_config;

		$campaigns = array();
		$where = "";
		$limitQuery = "";
		if ($limit && is_int($limit)) {
			if ($sugar_config['dbconfig']['db_type'] == 'mssql') {
				$limitQuery = "TOP ". (int) $limit;
			} else {
				$limitQuery = "LIMIT ". (int) $limit;
			}
		}
		if ($campaignIds && is_array($campaignIds)) {
			$where = " WHERE id IN ('".implode('\',\'', $campaignIds)."')";
		}
		if ($datetime) {
			$where .= ($where) ? " OR" : " WHERE";
			$where .= " (date_modified > '". $datetime. "')";
		}
		$fieldSelect = implode(', ', $fields);
		if ($sugar_config['dbconfig']['db_type'] == 'mssql') {
			$query = 'SELECT '.$limitQuery.' '.$fieldSelect.' FROM campaigns '.$where;
		} else {
			$query = 'SELECT '.$fieldSelect.' FROM campaigns '.$where.' '.$limitQuery;
		}

		$queryResult = $db->query($query);
		while (($queryRow=$db->fetchByAssoc($queryResult)) != null) {
			$campaigns[$queryRow['id']] = $queryRow;
		}
		return $campaigns;
	}
	/**
	 * Queries the database to generate a mapping of campaign ids to email addresses for Contacts/Leads.
	 * 
	 * @param array $campaignIds
	 * @param array $relevantRecords optional param for limiting the returned prospect emails
	 * @param string $relevantType type of record to which $relevantRecords refers, also optional
	 * @return array $campaignAssociationMap an array whose keys are campaign ids and whose values are
	 *      arrays containing all the emails from the corresponding $ids
	 */
	function getEmailMapFromCampaigns($campaignIds, $relevantRecords = null, $relevantType = null) {
		$campaignAssociationMap = array();
		foreach ($campaignIds as $campaignId) {
			$campaignAssociationMap[$campaignId] = array();
			$campaignLists = getListDataFromCampaignId($campaignId);

			// Get beans to include
			$includeContactBeans = array();
			$includeLeadBeans = array();
			foreach($campaignLists['default'] as $listData) {
				$beans = getBeanIdsFromProspectLists($listData['prospect_list_id'], $relevantRecords, $relevantType);
				$includeContactBeans = array_merge($includeContactBeans, $beans['contactBeanIds']);
				$includeLeadBeans = array_merge($includeLeadBeans, $beans['leadBeanIds']);
			}
			// Get beans to exclude
			$excludeContactBeans = array();
			$excludeLeadBeans = array();
			foreach($campaignLists['exempt'] as $listData) {
				$beans = getBeanIdsFromProspectLists($listData['prospect_list_id'], $relevantRecords, $relevantType);
				$excludeContactBeans = array_merge($excludeContactBeans, $beans['contactBeanIds']);
				$excludeLeadBeans = array_merge($excludeLeadBeans, $beans['leadBeanIds']);
			}
			// Get emails from include / exclude beans
			$contactBeans = array_diff($includeContactBeans, $excludeContactBeans);
			$leadBeans = array_diff($includeLeadBeans, $excludeLeadBeans);
			$emails = array_merge(array_keys(getEmailsFromBeanIds($contactBeans, 'Contacts')), array_keys(getEmailsFromBeanIds($leadBeans, 'Leads')));

			// Exclude emails based on domain
			$excludeDomains = array();
			foreach($campaignLists['exempt_domain'] as $key => $listData) {
				$excludeDomains[$key] = $listData['domain_name'];
			}
			if (count($excludeDomains)) {
				$emails = excludeEmailDomains($emails, $excludeDomains);
			}
			// Exclude email addresses
			$excludeEmailAddressListIds = array();
			foreach ($campaignLists['exempt_address'] as $exemptAddress) {
				$excludeEmailAddressListIds []= $exemptAddress['prospect_list_id'];
			}
			$emails = excludeEmailAddresses($emails, $excludeEmailAddressListIds);

			$campaignAssociationMap[$campaignId] = $emails;
		}
		return $campaignAssociationMap;
	}

	function excludeEmailDomains($emails, $domains) {
		if(!$domains) {
			return $emails;
		}
		
		$culledEmails = array();
		foreach ($emails as $email) {
			$excludeEmail = false;
			foreach ($domains as $excludeDomain) {
				if (strtolower(substr($email, strpos($email, '@') + 1)) === strtolower($excludeDomain)) {
					$excludeEmail = true;
					break;
				}
			}
			if(!$excludeEmail) {
				$culledEmails[] = $email;
			}
		}
		return $culledEmails;
	}

	function excludeEmailAddresses($emails, $exemptAddressListIds) {
		if(!$emails || !$exemptAddressListIds) {
			return $emails;
		}

		$limit = 1000;
		$offset = 0;
		do {
			$excludedBeanIds = getBeanIdsFromProspectLists($exemptAddressListIds, null, null, $limit, $offset);
			$offset += $limit;
			$emails = array_diff($emails, array_keys(getEmailsFromBeanIds($excludedBeanIds['leadBeanIds'], 'Leads')));
			$emails = array_diff($emails, array_keys(getEmailsFromBeanIds($excludedBeanIds['contactBeanIds'], 'Contacts')));
			if(count($emails) == 0) {
				break;
			}
		} while (count($excludedBeanIds['leadBeanIds']) || count($excludedBeanIds['contactBeanIds']));

		return $emails;
	}

	/**
	 * @param int $memberLimit
	 * @param array.<string> $campaignIds
	 * @param null $lastQueriedProspectIds Array that maps campaignId to a list of default listIds & the last
	 * 		prospectId (contact or lead) queried from this list
	 */
	function getCampaignMembershipDataInChunks($campaignIds, $memberLimit, $lastQueriedProspectIds = null) {
		$campaignReturnData = array();
		foreach($campaignIds as $campaignId) {
			// Collect data for this campaign
			$campaignReturnData[$campaignId] = array(
				'lastQueriedProspectId' => null,
				'prospect_emails' => array(),
				'finished' => false
			);

			// Get lists to query for included emails
			$campaignLists = getListDataFromCampaignId($campaignId);

			// Query default lists (defines prospects to be included) for lead / contact beanIds and organize data
			$lastId = isset($lastQueriedProspectIds[$campaignId]) ? $lastQueriedProspectIds[$campaignId] : null;
			$defaultListIds = array();
			foreach($campaignLists['default'] as $key => $listData) {
				$defaultListIds[$key] = $listData['prospect_list_id'];
			}
			$includeData = getChunkedBeanIdsFromProspectLists($defaultListIds, $memberLimit, $lastId);
			// Organized data
			$campaignReturnData[$campaignId]['finished'] = $includeData['finished'];
			$campaignReturnData[$campaignId]['lastQueriedProspectId'] = $includeData['lastBeanId'];
			$contactBeanIds = $includeData['contactBeanIds'];
			$leadBeanIds= $includeData['leadBeanIds'];

			// Filter out excluded beanIds and get emails
			$emails = array();
			$exemptListIds = array();
			foreach($campaignLists['exempt'] as $key => $listData) {
				$exemptListIds[$key] = $listData['prospect_list_id'];
			}

			if($contactBeanIds) {
				$excludedContactBeanIds = getBeanIdsFromProspectLists($exemptListIds, $contactBeanIds, 'Contacts');
				$contactBeanIds = array_diff($contactBeanIds, $excludedContactBeanIds['contactBeanIds']);
				$emails = array_keys(getEmailsFromBeanIds($contactBeanIds, 'Contacts'));
			}
			if($leadBeanIds) {
				$excludedLeadBeanIds = getBeanIdsFromProspectLists($exemptListIds, $leadBeanIds, 'Leads');
				$leadBeanIds = array_diff($leadBeanIds, $excludedLeadBeanIds['leadBeanIds']);
				$emails = array_merge($emails, array_keys(getEmailsFromBeanIds($leadBeanIds, 'Leads')));
			}

			// Exclude domains
			if($emails) {
				$domains = array();
				foreach($campaignLists['exempt_domain'] as $key => $listData) {
					$domains[$key] = $listData['domain_name'];
				}
				$emails = excludeEmailDomains($emails, $domains);
			}

			// Exclude email addresses
			$exemptAddressListIds = array();
			foreach($campaignLists['exempt_address'] as $key => $listData) {
				$exemptAddressListIds[$key] = $listData['prospect_list_id'];
			}
			$emails = excludeEmailAddresses($emails, $exemptAddressListIds);

			$campaignReturnData[$campaignId]['prospect_emails'] = $emails;
		}
		return $campaignReturnData;
	}

	/**
	 * @param string $campaignId
	 * @return array returns listData (prospect_list_id, list_type, domain_name) organized by list type
	 */
	function getListDataFromCampaignId($campaignId) {
		global $db;
		$listData = array(
			'exempt' => array(),
			'exempt_address' => array(),
			'exempt_domain' => array(),
			'default' => array(),
		);

		$listTypes = "('exempt','exempt_address', 'exempt_domain', 'default')";
		$where = "WHERE plc.prospect_list_id=pl.id AND plc.deleted=0 AND pl.deleted=0 AND campaign_id='$campaignId' AND pl.list_type in $listTypes";
		$queryResult = $db->query("SELECT plc.prospect_list_id, pl.list_type, pl.domain_name FROM prospect_list_campaigns plc, prospect_lists pl $where");

		while (($queryRow=$db->fetchByAssoc($queryResult)) != null) {
			// This logic purposefully ignores 'seed' and 'test' target lists,
			// neither of which should not be used to track leads
			$type = $queryRow['list_type'];
			if(!isset($listData[$type])) {
				$listData[$type] = array();
			}
			$listData[$type][] = $queryRow;
		}
		return $listData;
	}

	/**
	 * Queries the database to generate an array email addresses for a given prospect list.
	 *
	 * @param string | array.<string> $prospectListId
	 * @param array $relevantRecords optional param for limiting the returned prospect emails
	 * @param string $relevantType type of record to which $relevantRecords refers, also optional
	 * @return array containing leadBeanIds and contactBeanIds:  array('leadBeanIds' => array(), 'contactBeanIds' => array())
	 */
	function getBeanIdsFromProspectLists($prospectListIds, $relevantRecords = null, $relevantType = null, $limit = null, $offset = 0) {
		global $db;
		global $sugar_config;
		$usingMSSQL = $sugar_config['dbconfig']['db_type'] == 'mssql';

		// Setup return data
		$beanIds = array(
			'leadBeanIds' => array(),
			'contactBeanIds' => array(),
		);
		// If there's nothing to search through, return
		if(!$prospectListIds) {
			return $beanIds;
		}

		// Setup where clause
		$where = "WHERE deleted=0";
		if ($relevantRecords && is_array($relevantRecords)) {
			$where .= " AND (related_id IN ('".implode('\',\'', $relevantRecords)."') AND related_type='".$relevantType."')";
		}
		if(is_array($prospectListIds)) {
			$where .= " AND prospect_list_id in ('" . implode("','", $prospectListIds) . "')";
		} else {
			$where .= " AND prospect_list_id = '$prospectListIds'";
		}

		// Setup offset / limit
		$limitQuery = null;
		if ($limit && is_int($limit)) {
			$limit = (int) $limit;
			$offset = (int) $offset;
			if ($usingMSSQL) {
				$limitQuery = "ORDER BY id OFFSET $offset ROWS FETCH NEXT $limit";
			} else {
				$limitQuery = "ORDER BY id LIMIT $offset, $limit";
			}
		}

		$query = "SELECT related_id, related_type FROM prospect_lists_prospects $where $limitQuery";
		$queryResult = $db->query($query);
		while (($queryRow=$db->fetchByAssoc($queryResult)) != null) {
			if ($queryRow['related_type'] == 'Leads') {
				$beanIds['leadBeanIds'][] = $queryRow['related_id'];
			} elseif ($queryRow['related_type'] == 'Contacts') {
				$beanIds['contactBeanIds'][] = $queryRow['related_id'];
			}
		}

		return $beanIds;
	}

	/**
	 * @param string $prospectListIds The List we're querying from
	 * @param int $limit The maximum number of records to return
	 * @param string | null $lastId The last prospectId that was queried, so we can search past this id (query is ordered by related_id)
	 * @return array
	 */
	function getChunkedBeanIdsFromProspectLists($prospectListIds, $limit, $lastId = null) {
		global $db, $sugar_config;
		$usingMSSQL = $sugar_config['dbconfig']['db_type'] == 'mssql';

		$prospectListIds = (array) $prospectListIds;
		if(!$prospectListIds) {
			serializeError("No lists provided.");
		}

		$beanIds = array(
			'leadBeanIds' => array(),
			'contactBeanIds' => array(),
			'lastBeanId' => null,
			'finished' => false,
		);

		$where = "WHERE deleted=0";
		if ($lastId) {
			$where .= " AND related_id > '$lastId'";
		}
		$listIds = implode("','", $prospectListIds);
		$where .= " AND prospect_list_id IN ('$listIds')";

		$limitQuery = null;
		if ($limit && is_int($limit)) {
			if ($usingMSSQL) {
				$limitQuery = "TOP ". (int) ($limit);
			} else {
				$limitQuery = "LIMIT ". (int) ($limit);
			}
		}

		$query = null;
		if ($usingMSSQL) {
			$query = "SELECT $limitQuery related_id, related_type FROM prospect_lists_prospects $where ORDER BY related_id";
		} else {
			$query = "SELECT related_id, related_type FROM prospect_lists_prospects $where ORDER BY related_id $limitQuery";
		}
		$queryResult = $db->query($query);

		$count = 0;
		while (($queryRow=$db->fetchByAssoc($queryResult)) != null) {
			if ($queryRow['related_type'] == 'Leads') {
				$beanIds ['leadBeanIds'][] = $queryRow['related_id'];
			} elseif ($queryRow['related_type'] == 'Contacts') {
				$beanIds ['contactBeanIds'][] = $queryRow['related_id'];
			}
			$beanIds ['lastBeanId'] = $queryRow['related_id'];
			$count++;
		}
		// We're finished querying this list when number of records returned is less than the limit
		$beanIds ['finished'] = $count < $limit;
		return $beanIds;
	}

