<?php
	include_once('common.php');

	$oppIdsToContactEmails = getOpportunityIds($_POST['emails'], 'Contacts');
	$oppIdsToLeadEmails = getOpportunityIds($_POST['emails'], 'Leads');
	$allOppIds = array_merge(array_keys($oppIdsToContactEmails), array_keys($oppIdsToLeadEmails));

	//generate an array from the opportunity ids associated with the contact/lead
	//emails where the key is the email address and the value looks like:
	//	<email1> => {
	//		'prospect' => <email1>,
	//		'opportunities' => {
	//			<opp1_id> => {
	//				{ 'name'=> 'name', 'value'=> <opp1_name> },
	//				{...}
	//			{...}
	//		},
	//	},
	//	{...}
	// }
	//

	$opps = array();
	
	$classSeed = new Opportunity();
	foreach ($allOppIds as $oppId) {
		$classObject = $classSeed->retrieve($oppId);
		if (!$classObject) {
			continue;
		}
		
		$oppNameValues = getNameValueListFromObject($classObject);
		$allEmails = array_merge((array)$oppIdsToContactEmails[$oppId], (array)$oppIdsToLeadEmails[$oppId]);
		foreach ($allEmails as $email) {
			if (!array_key_exists($email, $opps)) {
				$opps[$email] = array('prospect' => $email, 'opportunities' => array());
			}
			$opps[$email]['opportunities'][$oppId] = $oppNameValues;
		}
	}

	serializeResult(array('entry_list' => $opps));
	
	/**
	 * Queries the database to generate a mapping of opportunity ids to email addresses for Contacts.
	 * 
	 * @param array $emails the email addresses to pull in
	 * @param string $module the module to query over
	 * @return array an array whose keys are opportunity ids and whose values are arrays containing all the
	 *			emails from the $emails passed in which correspond to $modules associated with that opportunity
	 */
	function getOpportunityIds($emails, $module)
	{
		global $db;
		$beanIdToEmailMap = getBeanIdsFromEmails($emails, $module);
		if (!$beanIdToEmailMap) {
			return array();
		}
		
		$singletonModule = substr(strtolower($module), 0, -1);
		if ($module == 'Contacts') {
			$foreignId = $singletonModule.'_id';
			$joinTable = 'opportunities_'.$singletonModule.'s';
		} elseif ($module == 'Leads') {
			$foreignId = 'id';
			$joinTable = 'leads';
		}
		$queryResult = $db->query('SELECT '.$foreignId.', opportunity_id FROM '.$joinTable.' WHERE '.$foreignId.' IN (\''.implode('\', \'', array_keys($beanIdToEmailMap)).'\')');
		$opportunityIdToEmails = array();
		while (($queryRow=$db->fetchByAssoc($queryResult)) != null) {
			$oppId = $queryRow['opportunity_id'];
			if (!array_key_exists($oppId, $opportunityIdToEmails)) {
				$opportunityIdToEmails[$oppId] = array();
			}
			$opportunityIdToEmails[$oppId][] = $beanIdToEmailMap[$queryRow[$foreignId]];
		}
		return $opportunityIdToEmails;
	}
?>
