<?php
function pre_install()
{
	global $current_user;
	
	require_once('include/utils.php');
	require_once('include/utils/file_utils.php');
	require_once('config.php');
	require_once('include/MVC/Controller/SugarController.php');
	require_once('modules/ModuleBuilder/controller.php');
	require_once('modules/ModuleBuilder/parsers/ParserFactory.php');

	$fields = array(
		'pi_campaign_c',
		'pi_score_c',
		'pi_grade_c',
		'pi_url_c',
		'pi_created_date_c',
		'pi_comments_c',
		'pi_first_activity_c',
		'pi_last_activity_c',
		'pi_first_referrer_c',
		'pi_first_search_term_c',
		'pi_first_search_type_c',
		'pi_notes_c',
		'pi_opt_out_c',
		'pi_do_not_email_c',
		'pi_html_c',
		'pi_html_lists_c',
		'pi_html_social_c',
		'pi_utm_campaign_c',
		'pi_utm_medium_c',
		'pi_utm_term_c',
		'pi_utm_source_c',
		'pi_utm_content_c',
		'pi_is_queued_c',
	);

	$layoutFields = array();
	foreach ($fields as $field) {
		$layoutFields[$field] = array('name' => $field);
	}
	
	$url = 'https://pi.pardot.com/prospect/activity/source/sugar/email/{$bean->email1}{$current_user->pardot_auth}';
	$layoutFields ['pi_html_c'] = array(
		'name'			=> 'pi_html_c',
		'customCode'	=> '<iframe src=\''.$url.'\' width=\'100%\' height=\'250px\' marginwidth=\'0\' marginheight=\'0\' scrolling=\'no\' frameborder=\'0\'></iframe>'
	);

    $url = 'https://pi.pardot.com/listProspect/crm/source/sugar/email/{$bean->email1}{$current_user->pardot_auth}';
	$layoutFields ['pi_html_lists_c'] = array(
		'name'			=> 'pi_html_lists_c',
		'customCode'	=> '<iframe src=\''.$url.'\' width=\'100%\' height=\'250px\' marginwidth=\'0\' marginheight=\'0\' scrolling=\'yes\' frameborder=\'0\'></iframe>'
	);

	$url = 'https://pi.pardot.com/qwerly/socialData/source/sugar/email/{$bean->email1}{$current_user->pardot_auth}';
	$layoutFields ['pi_html_social_c'] = array(
		'name'			=> 'pi_html_social_c',
		'customCode'	=> '<iframe src=\''.$url.'\' width=\'100%\' height=\'250px\' marginwidth=\'0\' marginheight=\'0\' scrolling=\'yes\' frameborder=\'0\'></iframe>'
	);

	addFieldDetailView('Contacts', $layoutFields);
	addFieldDetailView('Leads', $layoutFields);  

    // this is so that the Pardot tab / module will be displayed...
    // the next time the return_app_list_strings_language() method in Sugar's include/utils.js file
    // is called, this value will be repopulated
    sugar_cache_clear('app_list_strings.en_us'); 
}


/**
 * adds a custom field defined in manifest.php to a module's custom detailviewdefs.php
 * @param string module module name where fields will be added
 * @param array layoutFields contains the layout property definitions to be added to detailviewdefs.php 
 */

function addFieldDetailView ($module, $layoutFields) {
	echo "<br /><strong> Updating the $module module with Pardot fields.</strong><br />";	

	$parser = ParserFactory::getParser('detailview', $module);
    
	foreach ($parser->_viewdefs['panels'] as $panelName => $panel) {
		foreach($panel as $rowIndex => $panelRow) {
			foreach($panelRow as $columnIndex => $panelColumn) {
				if (is_array($panelColumn) && isset($panelColumn['name'])) {
					$fieldName = $panelColumn['name'];
				} else {
					$fieldName = $panelColumn;
				}
				
				if(array_key_exists($fieldName, $layoutFields)) {
					$parser->_viewdefs['panels'][$panelName][$rowIndex][$columnIndex] = $layoutFields[$fieldName];
					echo ("<strong>$fieldName</strong> field updated in $module Detail View.<br />");
					unset($layoutFields[$fieldName]);
				}
			}
		}
	}

	$sixFivePlus = false;
	if(class_exists('SugarConfig') && method_exists('SugarConfig', 'getInstance')) {
		$version = SugarConfig::getInstance()->get('sugar_version');
		// booby trap! should last us a few years...
		if($version && (strpos($version, "6.5") === 0 || strpos($version, "6.6") === 0 || strpos($version, "6.7") === 0 || strpos($version, "6.8") === 0)) {
			$sixFivePlus = true;
		}
	}

	foreach ($layoutFields as $panelName => $panelDef) {
		if($sixFivePlus) {
			$parser->addField($panelDef, $parser->_viewdefs['panels']['Pardot']);
		} else {
			$parser->_viewdefs['panels']['Pardot'] []= array($panelDef);	
		}
		
		echo ("<strong>$panelName</strong> field added to $module Detail View.<br />");
	}

	$parser->handleSave(false);
}
